@@ -47,6 +47,7 @@
 #include <ktabwidget.h>
 #include <kedittoolbar.h>
 #include <ksqueezedtextlabel.h>
+#include <kpushbutton.h>
 
 #include <kstdaccel.h>
 #include <kaction.h>
@@ -54,6 +55,7 @@
 
 #include <interfaces/torrentinterface.h>
 #include <torrent/peermanager.h>
+#include <torrent/chunkmanager.h>
 #include <torrent/uploadcap.h>
 #include <torrent/downloadcap.h>
 #include <util/error.h>
@@ -79,122 +81,94 @@
 #include "torrentcreatordlg.h"
 #include "pastedialog.h"
 #include "queuedialog.h"
+#include "ipfilterwidget.h"
 #include <util/functions.h>
 #include <interfaces/functions.h>
 #include <interfaces/plugin.h>
 #include <interfaces/prefpageinterface.h>
 #include <expandablewidget.h>
 #include <pluginmanager.h>
+#include <groups/group.h>
+#include <groups/groupview.h>
+#include <mse/streamsocket.h>
+
+
+
+namespace kt
+{
+	QString DataDir();
+}
+
+#include <util/profiler.h>
+
 
 
 using namespace bt;
 using namespace kt;
 
 KTorrent::KTorrent()
-		: KMainWindow( 0, "KTorrent" ),
-		m_view(0), m_seedView(0), m_systray_icon(0)
+	: KMdiMainFrm( 0, "KTorrent",KMdi::IDEAlMode ),m_group_view(0),
+		m_view(0), m_systray_icon(0)
 {
-	bool debug = bt::Globals::instance().isDebugModeSet();
+	setHidden(true);
+	setToolviewStyle(KMdi::TextAndIcon);
+	
 	KIconLoader* iload = KGlobal::iconLoader();
-
-	m_tabs = new KTabWidget(0);
+	
 	m_view = new KTorrentView(0);
-	m_seedView = new KTorrentView(0, true);
-	m_exp = new ExpandableWidget(m_tabs,this);
-	m_view_exp = new ExpandableWidget(m_view,m_tabs);
-	m_seedView_exp = new ExpandableWidget(m_seedView,m_tabs);
-	m_tabs->addTab(m_view_exp,iload->loadIconSet("down", KIcon::Small),i18n("Downloads"));
-	m_tabs->addTab(m_seedView_exp,iload->loadIconSet("up", KIcon::Small),i18n("Uploads"));
+	m_view_exp = new ExpandableWidget(m_view,0);
+	addTabPage(m_view_exp,iload->loadIconSet("folder", KIcon::Small),i18n("Torrents"));
+	
+	m_group_view = new kt::GroupView(m_view,actionCollection());
+	m_group_view->loadGroups();
 
 	m_pref = new KTorrentPreferences(*this);
 	m_core = new KTorrentCore(this);
 	m_systray_icon = new TrayIcon(m_core, this);
 	
-	connect(m_tabs, SIGNAL(currentChanged(QWidget*)), this, SLOT(currentTabChanged( QWidget* )));
+	connect(m_group_view,SIGNAL(currentGroupChanged( kt::Group* )),
+			this,SLOT(groupChanged(kt::Group*)));
 	
-	connect(m_core, SIGNAL(finished( kt::TorrentInterface* )), m_view, SLOT(torrentFinished( kt::TorrentInterface* )));
+	connect(m_group_view,SIGNAL(groupRenamed(kt::Group*)),
+			this,SLOT(groupChanged(kt::Group*)));
 	
-	connect(m_view, SIGNAL(viewChange( kt::TorrentInterface* )), m_seedView, SLOT(addTorrent( kt::TorrentInterface* )));
-	connect(m_seedView, SIGNAL(viewChange( kt::TorrentInterface* )), m_view, SLOT(addTorrent( kt::TorrentInterface* )));
-	connect(m_seedView, SIGNAL(viewChange( kt::TorrentInterface* )), m_systray_icon, SLOT(viewChanged( kt::TorrentInterface* )));
+	connect(m_core, SIGNAL(finished( kt::TorrentInterface* )), 
+			m_view, SLOT(torrentFinished( kt::TorrentInterface* )));
 
 	connect(m_core,SIGNAL(torrentAdded(kt::TorrentInterface* )),
 			m_view,SLOT(addTorrent(kt::TorrentInterface* )));
-	
-	connect(m_core,SIGNAL(torrentAdded(kt::TorrentInterface* )),
-			m_seedView,SLOT(addTorrent(kt::TorrentInterface* )));
 
 	connect(m_core,SIGNAL(torrentRemoved(kt::TorrentInterface* )),
 			m_view,SLOT(removeTorrent(kt::TorrentInterface* )));
-	
-	connect(m_core,SIGNAL(torrentRemoved(kt::TorrentInterface* )),
-			m_seedView,SLOT(removeTorrent(kt::TorrentInterface* )));
 
 	connect(m_view,SIGNAL(currentChanged(kt::TorrentInterface* )),
-			this,SLOT(currentDownloadChanged(kt::TorrentInterface* )));
-	
-	connect(m_seedView,SIGNAL(currentChanged(kt::TorrentInterface* )),
-			this,SLOT(currentSeedChanged(kt::TorrentInterface* )));
+			this,SLOT(currentTorrentChanged(kt::TorrentInterface* )));
 
 	connect(m_view,SIGNAL(wantToRemove(kt::TorrentInterface*,bool )),
 			m_core,SLOT(remove(kt::TorrentInterface*,bool )));
-	
-	connect(m_seedView,SIGNAL(wantToRemove(kt::TorrentInterface*,bool )),
-			m_core,SLOT(remove(kt::TorrentInterface*,bool )));
 
 	connect(m_view,SIGNAL(dropped(QDropEvent*,QListViewItem*)),
 			this,SLOT(urlDropped(QDropEvent*,QListViewItem*)));
 	
-	connect(m_seedView,SIGNAL(dropped(QDropEvent*,QListViewItem*)),
-			this,SLOT(urlDropped(QDropEvent*,QListViewItem*)));
-	
 	connect(m_view,SIGNAL(wantToStart( kt::TorrentInterface* )),
 			m_core,SLOT(start( kt::TorrentInterface* )));
 	
-	connect(m_seedView,SIGNAL(wantToStart( kt::TorrentInterface* )),
-			m_core,SLOT(start( kt::TorrentInterface* )));
-	
 	connect(m_view,SIGNAL(wantToStop( kt::TorrentInterface*, bool )),
 			m_core,SLOT(stop( kt::TorrentInterface*, bool )));
 	
-	connect(m_seedView,SIGNAL(wantToStop( kt::TorrentInterface*, bool )),
-			m_core,SLOT(stop( kt::TorrentInterface*, bool )));
+	connect(m_view,SIGNAL(needsDataCheck( kt::TorrentInterface* )),
+			m_core,SLOT(doDataCheck( kt::TorrentInterface* )));
 	
 	connect(m_view,SIGNAL(updateActions( bool, bool, bool, bool )),
 			this,SLOT(onUpdateActions( bool, bool, bool, bool )));
 	
-	connect(m_seedView,SIGNAL(updateActions( bool, bool, bool, bool )),
-			this,SLOT(onUpdateActions( bool, bool, bool, bool )));
-	
-	
 	//connect Core queue() with queue() from KTView.
-	connect(m_view, SIGNAL(queue( kt::TorrentInterface* )), m_core, SLOT(queue( kt::TorrentInterface* )));
-	connect(m_seedView, SIGNAL(queue( kt::TorrentInterface* )), m_core, SLOT(queue( kt::TorrentInterface* )));
-
-	// then, setup our actions
-	setupActions();
-
-	// and a status bar
-	statusBar()->show();
-
-	// apply the saved mainwindow settings, if any, and ask the mainwindow
-	// to automatically save settings if changed: window size, toolbar
-	// position, icon size, etc.
-	setAutoSaveSettings();
-
-	currentDownloadChanged(0);
-	applySettings(false);
-
-	setCentralWidget(m_exp);
-	if (debug)
-	{
-		//m_exp->expand(new LogViewer(bt::Globals::instance().getLog(),0),kt::BELOW);
-	}
-
-	m_dcop = new KTorrentDCOP(this);
-
-	m_core->loadTorrents();
-	setStandardToolBarMenuEnabled(true);
+	connect(m_view, SIGNAL(queue( kt::TorrentInterface* )), 
+			m_core, SLOT(queue( kt::TorrentInterface* )));
+	
+	connect(m_core, SIGNAL(torrentRemoved( kt::TorrentInterface* )),
+			m_group_view, SLOT(onTorrentRemoved( kt::TorrentInterface* )));
+	
 
 	m_statusInfo = new KSqueezedTextLabel(this);
 	m_statusSpeed = new QLabel(this);
@@ -208,10 +182,22 @@ KTorrent::KTorrent()
 	statusBar()->addWidget(m_statusSpeed);
 	statusBar()->addWidget(m_statusTransfer);
 
+	
+	setupActions();
+	currentTorrentChanged(0);
+	
+	m_dcop = new KTorrentDCOP(this);
+
+	setStandardToolBarMenuEnabled(true);
+
 	QToolTip::add(m_statusInfo, i18n("Info"));
 	QToolTip::add(m_statusTransfer, i18n("Data transferred during the current session"));
 	QToolTip::add(m_statusSpeed, i18n("Current speed of all torrents combined"));
 	
+	//first apply settings..
+	applySettings(false);
+	//then load stuff from core
+	m_core->loadTorrents();
 	m_core->loadPlugins();
 
 	connect(&m_gui_update_timer, SIGNAL(timeout()), this, SLOT(updatedStats()));
@@ -232,34 +218,93 @@ KTorrent::KTorrent()
 			val = 500;
 	}
 	m_gui_update_timer.start(val);
-
+	
+	groupChanged(m_group_view->currentGroup());
+	tabWidget()->setHoverCloseButton(false);
+	
+	m_close_cur_tab = new KPushButton(KGuiItem(QString::null,"tab_remove"),tabWidget());
+	connect(m_close_cur_tab,SIGNAL(clicked()),this,SLOT(tabClosePressed()));
+	tabWidget()->setCornerWidget(m_close_cur_tab,Qt::TopRight);
+	connect(tabWidget(),SIGNAL(currentChanged(QWidget*)),this,SLOT(currentTabChanged( QWidget* )));
+	statusBar()->show();
+	
+	addToolWidget(m_group_view,"player_playlist",i18n("Groups"),DOCK_LEFT);
+	
+	setAutoSaveSettings("WindowStatus",true);
+	KGlobal::config()->setGroup("WindowStatus");
 	bool hidden_on_exit = KGlobal::config()->readBoolEntry("hidden_on_exit",false);
-	if (!(Settings::showSystemTrayIcon() && hidden_on_exit))
+	if (Settings::showSystemTrayIcon())
+	{
+		if (hidden_on_exit)
+		{
+			Out(SYS_GEN|LOG_DEBUG) << "Starting minimized" << endl;
+			hide();
+		}
+		else
+		{
+			show();
+		}
+	}
+	else
 	{
-		Out(SYS_GEN|LOG_DEBUG) << "Showing KT" << endl;
 		show();
 	}
+	MaximizeLimits();
 }
 
 KTorrent::~KTorrent()
 {
+	while (m_tab_map.count() > 0)
+	{
+		QMap<QWidget*,Tab>::iterator itr = m_tab_map.begin();
+		removeTabPage(itr.key());
+	}
 	delete m_dcop;
 	delete m_core;
 	delete m_pref;
 	delete m_statusInfo;
 	delete m_statusTransfer;
 	delete m_statusSpeed;
+	delete m_view_exp;
 }
 
-void KTorrent::addTabPage(QWidget* page,const QIconSet & icon,const QString & caption)
+void KTorrent::groupChanged(kt::Group* g)
 {
-	m_tabs->addTab(page,icon,caption);
-	page->show();
+	if (!m_tab_map.contains(m_view_exp))
+		return;
+	
+	KMdiChildView* cv = m_tab_map[m_view_exp].child;
+	if (cv)
+	{
+		cv->setCaption(g->groupName());
+		cv->setMDICaption(g->groupName());
+		cv->setTabCaption(g->groupName());
+		cv->setIcon(g->groupIcon());
+	}
+	m_view->setCurrentGroup(g);
+}
+
+void KTorrent::addTabPage(QWidget* page,const QIconSet & icon,
+						  const QString & caption,kt::CloseTabListener* ctl)
+{
+	KMdiChildView* cv = createWrapper(page,caption,caption);
+	addWindow(cv);
+	cv->setIcon(icon.pixmap(QIconSet::Small,QIconSet::Active));
+	Tab t;
+	t.child = cv;
+	t.ctl = ctl;
+	m_tab_map.insert(page,t);
 }
 
 void KTorrent::removeTabPage(QWidget* page)
 {
-	m_tabs->removePage(page);
+	if (!m_tab_map.contains(page))
+		return;
+	
+	KMdiChildView* cv = m_tab_map[page].child;
+	m_tab_map.erase(page);
+	page->reparent(0,QPoint());
+	closeWindow(cv);
 }
 
 void KTorrent::addPrefPage(PrefPageInterface* page)
@@ -283,6 +328,11 @@ void KTorrent::applySettings(bool change_port)
 	net::SocketMonitor::setDownloadCap(Settings::maxDownloadRate()*1024);
 	net::SocketMonitor::setUploadCap(Settings::maxUploadRate()*1024);
 	m_core->setKeepSeeding(Settings::keepSeeding());
+	mse::StreamSocket::setTOS(Settings::typeOfService());
+	if (Settings::allwaysDoUploadDataCheck())
+		ChunkManager::setMaxChunkSizeForDataCheck(0);
+	else
+		ChunkManager::setMaxChunkSizeForDataCheck(Settings::maxSizeForUploadDataCheck() * 1024);
 
 	if (Settings::showSystemTrayIcon())
 	{
@@ -302,6 +352,8 @@ void KTorrent::applySettings(bool change_port)
 	
 	if (Settings::useExternalIP())
 		Tracker::setCustomIP(Settings::externalIP());
+	else
+		Tracker::setCustomIP(QString::null);
         
 	Downloader::setMemoryUsage(Settings::memoryUsage());
 	Choker::setNumUploadSlots(Settings::numUploadSlots());
@@ -329,7 +381,7 @@ void KTorrent::applySettings(bool change_port)
 	dht::DHTBase & ht = Globals::instance().getDHT();
 	if (Settings::dhtSupport() && !ht.isRunning())
 	{
-		ht.start(KGlobal::dirs()->saveLocation("data","ktorrent") + "dht_table",Settings::dhtPort());
+		ht.start(kt::DataDir() + "dht_table",Settings::dhtPort());
 	}
 	else if (!Settings::dhtSupport() && ht.isRunning())
 	{
@@ -339,7 +391,7 @@ void KTorrent::applySettings(bool change_port)
 	{
 		Out(SYS_GEN|LOG_NOTICE) << "Restarting DHT with new port " << Settings::dhtPort() << endl;
 		ht.stop();
-		ht.start(KGlobal::dirs()->saveLocation("data","ktorrent") + "dht_table",Settings::dhtPort());
+		ht.start(kt::DataDir() + "dht_table",Settings::dhtPort());
 	}
 	
 	if (Settings::useEncryption())
@@ -371,15 +423,11 @@ void KTorrent::onUpdateActions(bool can_start,bool can_stop,bool can_remove,bool
 	m_datacheck->setEnabled(can_scan);
 }
 
-void KTorrent::currentDownloadChanged(kt::TorrentInterface* tc)
+void KTorrent::currentTorrentChanged(kt::TorrentInterface* tc)
 {
-	notifyDownloadViewListeners(tc);
+	notifyViewListeners(tc);
 }
 
-void KTorrent::currentSeedChanged(kt::TorrentInterface* tc)
-{
-	notifySeedViewListeners(tc);
-}
 
 void KTorrent::setupActions()
 {
@@ -430,6 +478,11 @@ void KTorrent::setupActions()
 			"player_playlist", 0, this, SLOT(queueAction()),
 			actionCollection(), "queue_action");
 	
+	m_ipfilter = new KAction(
+			i18n("IPFilter"),
+			"filter", 0, this, SLOT(showIPFilter()),
+			actionCollection(), "ipfilter_action");
+	
 	m_datacheck = new KAction(
 			i18n("Check Data Integrity"),
 	QString::null,0,this,SLOT(checkDataIntegrity()),actionCollection(),"check_data");
@@ -449,7 +502,7 @@ void KTorrent::setupActions()
 
 	pref->plug(m_systray_icon->contextMenu());
 	
-	createGUI();
+	createGUI(0);
 }
 
 
@@ -469,15 +522,20 @@ bool KTorrent::queryClose()
 
 bool KTorrent::queryExit()
 {
+#ifdef KT_PROFILE
+	bt::Profiler::instance().saveToFile(kt::DataDir() + "profile");
+#endif
+	m_group_view->saveGroups();
 	// stop timers to prevent update
 	m_gui_update_timer.stop();
+	
+	KGlobal::config()->setGroup("WindowStatus");
+	KGlobal::config()->writeEntry( "hidden_on_exit",this->isHidden());
+	m_view->saveSettings();
+	hide();
 	m_core->onExit();
 	if (Globals::instance().getDHT().isRunning())
 		Globals::instance().getDHT().stop();
-	
-	KGlobal::config()->writeEntry( "hidden_on_exit",this->isHidden());
-	m_view->saveSettings();
-	m_seedView->saveSettings();
 	return true;
 }
 
@@ -514,92 +572,42 @@ void KTorrent::queueManagerShow()
 
 void KTorrent::startDownload()
 {
-	KTorrentView* curr_view = getCurrentView();
-	curr_view->startDownloads();
-			
-	TorrentInterface* tc = curr_view->getCurrentTC();
+	m_view->startDownloads();
+	TorrentInterface* tc = m_view->getCurrentTC();
 	
-	if(getCurrentPanel() == DOWNLOAD_VIEW)
-	{
-		currentDownloadChanged(tc);
-		m_view->onSelectionChanged(); // trigger an updateActions signal
-	}
-	
-	if(getCurrentPanel() == SEED_VIEW)
-	{
-		currentSeedChanged(tc);
-		m_seedView->onSelectionChanged(); // trigger an updateActions signal
-	}
+	currentTorrentChanged(tc);
+	m_view->onSelectionChanged(); // trigger an updateActions signal
 }
 
 void KTorrent::startAllDownloads()
 {
-	kt::PanelView what = getCurrentPanel();
-	m_core->startAll(((int)what)+1);
-	
-	if(getCurrentPanel() == DOWNLOAD_VIEW)
-	{
-		m_view->onSelectionChanged(); // trigger an updateActions signal
-	}
-	if(getCurrentPanel() == SEED_VIEW)
-	{
-		m_seedView->onSelectionChanged(); // trigger an updateActions signal
-	}
+	m_view->startAllDownloads();
+	m_view->onSelectionChanged(); // trigger an updateActions signal
 }
 
 void KTorrent::stopDownload()
 {
-	KTorrentView* curr_view = getCurrentView();
-	curr_view->stopDownloads();
+	m_view->stopDownloads();
 			
-	TorrentInterface* tc = curr_view->getCurrentTC();
-	
-	if(getCurrentPanel() == DOWNLOAD_VIEW)
-	{
-		currentDownloadChanged(tc);
-		m_view->onSelectionChanged(); // trigger an updateActions signal
-	}
-	
-	if(getCurrentPanel() == SEED_VIEW)
-	{
-		currentSeedChanged(tc);
-		m_seedView->onSelectionChanged(); // trigger an updateActions signal
-	}
+	TorrentInterface* tc = m_view->getCurrentTC();	
+	currentTorrentChanged(tc);
+	m_view->onSelectionChanged(); // trigger an updateActions signal
 }
 
 void KTorrent::stopAllDownloads()
 {
-	kt::PanelView what = getCurrentPanel();
-	m_core->stopAll(((int)what)+1);
-	
-	if(getCurrentPanel() == DOWNLOAD_VIEW)
-	{
-		m_view->onSelectionChanged(); // trigger an updateActions signal
-	}
-	if(getCurrentPanel() == SEED_VIEW)
-	{
-		m_seedView->onSelectionChanged(); // trigger an updateActions signal
-	}
+	m_view->stopAllDownloads();
+	m_view->onSelectionChanged(); // trigger an updateActions signal
 }
 
 void KTorrent::removeDownload()
 {
-	KTorrentView* curr_view = getCurrentView();
-	curr_view->removeDownloads();
+	m_view->removeDownloads();
 			
-	TorrentInterface* tc = curr_view->getCurrentTC();
-	if(getCurrentPanel() == DOWNLOAD_VIEW)
-	{
-		currentDownloadChanged(tc);
-		notifyDownloadViewListeners(tc);
-		m_view->onSelectionChanged(); // trigger an updateActions signal
-	}
-	if(getCurrentPanel() == SEED_VIEW)
-	{
-		currentSeedChanged(tc);
-		notifySeedViewListeners(tc);
-		m_seedView->onSelectionChanged(); // trigger an updateActions signal
-	}
+	TorrentInterface* tc = m_view->getCurrentTC();
+	currentTorrentChanged(tc);
+	notifyViewListeners(tc);
+	m_view->onSelectionChanged(); // trigger an updateActions signal
 }
 
 void KTorrent::optionsShowStatusbar()
@@ -638,7 +646,7 @@ void KTorrent::newToolbarConfig()
 {
 	// this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
 	// recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
-	createGUI();
+	createGUI(0);
 
 #if defined(KDE_MAKE_VERSION)
 # if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
@@ -663,7 +671,6 @@ void KTorrent::changeStatusbar(const QString& text)
 	// display the text on the statusbar
 	//statusBar()->message(text);
 	m_statusInfo->setText(text);
-	
 }
 
 void KTorrent::changeCaption(const QString& text)
@@ -705,38 +712,52 @@ void KTorrent::urlDropped(QDropEvent* event,QListViewItem*)
 
 void KTorrent::updatedStats()
 {
+	KT_PROF_START("updatedStats");
+	KT_PROF_START("misc");
 	m_startall->setEnabled(m_core->getNumTorrentsNotRunning() > 0);
 	m_stopall->setEnabled(m_core->getNumTorrentsRunning() > 0);
 	
 	CurrentStats stats = this->m_core->getStats();
 	
 	//m_statusInfo->setText(i18n("Some info here e.g. connected/disconnected"));
-	QString tmp = i18n("Speed up: %1 / down: %2")
-			.arg(KBytesPerSecToString((double)stats.upload_speed/1024.0))
-			.arg(KBytesPerSecToString((double)stats.download_speed/1024.0));
+	QString tmp = i18n("Speed down: %1 / up: %2")
+			.arg(KBytesPerSecToString((double)stats.download_speed/1024.0))
+			.arg(KBytesPerSecToString((double)stats.upload_speed/1024.0));
 
 	m_statusSpeed->setText(tmp);
 
-	QString tmp1 = i18n("Transferred up: %1 / down: %2")
-			.arg(BytesToString(stats.bytes_uploaded))
-			.arg(BytesToString(stats.bytes_downloaded));
+	QString tmp1 = i18n("Transferred down: %1 / up: %2")
+			.arg(BytesToString(stats.bytes_downloaded))
+			.arg(BytesToString(stats.bytes_uploaded));
 	m_statusTransfer->setText(tmp1);
-
-	m_view->update();
-	m_seedView->update();
-	m_systray_icon->updateStats(stats);
-	m_core->getPluginManager().updateGuiPlugins();
+	KT_PROF_END();
 	
+	KT_PROF_START("view");
+	m_view->update();
+	KT_PROF_END();
+	
+	KT_PROF_START("systray");
+	m_systray_icon->updateStats(stats,Settings::showSpeedBarInTrayIcon(),Settings::downloadBandwidth(), Settings::uploadBandwidth());
+	KT_PROF_END();
+	
+	KT_PROF_START("plugins");
+	m_core->getPluginManager().updateGuiPlugins();
+	KT_PROF_END();
+	
+#if 0
+
 	//update tab labels
 	QString tabText = i18n("Downloads %1/%2").arg(m_core->getNumRunning(true)).arg(m_core->countDownloads());
 	//kdDebug() << "tabtext: " << tabText << " " << m_tabs->tabLabel(m_view_exp) << endl;
-	if (tabText != m_tabs->tabLabel(m_view_exp).replace('&', ""))
-		m_tabs->setTabLabel(m_view_exp, tabText);
+	if (tabText != tabWidget()->tabLabel(m_view_exp).replace('&', ""))
+		tabWidget()->setTabLabel(m_view_exp, tabText);
 		
 	tabText = i18n("Uploads %1/%2").arg(m_core->getNumRunning(false,true)).arg(m_core->countSeeds());
 	if (tabText != m_tabs->tabLabel(m_seedView_exp).replace('&', ""))
 		m_tabs->setTabLabel(m_seedView_exp, tabText);
-	
+#endif
+
+	KT_PROF_START("DHT");
 	if (Globals::instance().getDHT().isRunning())
 	{
 		const dht::Stats & s = Globals::instance().getDHT().getStats();
@@ -745,6 +766,8 @@ void KTorrent::updatedStats()
 	}
 	else
 		m_statusDHT->setText(i18n("DHT: off"));
+	KT_PROF_END();
+	KT_PROF_END();
 }
 
 void KTorrent::mergePluginGui(Plugin* p)
@@ -773,32 +796,47 @@ void KTorrent::removeWidgetFromView(QWidget* w)
 	m_view_exp->remove(w);
 }
 
-void KTorrent::addWidgetInSeedView( QWidget * w, kt::Position pos )
+void KTorrent::addWidgetBelowView(QWidget* w,const QString & icon,const QString & caption)
 {
-	if (!w) return;
-	
-	m_seedView_exp->expand(w,pos);
-}
-
-void KTorrent::removeWidgetFromSeedView( QWidget * w )
-{
-	if (!w) return;
-	
-	m_seedView_exp->remove(w);
-}
-
-void KTorrent::addWidgetBelowView(QWidget* w)
-{
-	if (!w) return;
-	
-	m_exp->expand(w,kt::BELOW);
+	addToolWidget(w,icon,caption,DOCK_BOTTOM);
 }
 	
 void KTorrent::removeWidgetBelowView(QWidget* w)
+{
+	removeToolWidget(w);
+}
+
+void KTorrent::addToolWidget(QWidget* w,const QString & icon,const QString & caption,ToolDock dock)
 {
 	if (!w) return;
 	
-	m_exp->remove(w);
+	KMdiToolViewAccessor* tw = createToolWindow();
+	if (!icon.isNull())
+		w->setIcon(KGlobal::iconLoader()->loadIcon(icon,KIcon::Small));
+	tw->setWidgetToWrap(w,caption,caption);
+	switch (dock)
+	{
+		case DOCK_BOTTOM:
+			tw->place(KDockWidget::DockBottom,getMainDockWidget(),25);
+			break;
+			
+		case DOCK_LEFT:
+			tw->place(KDockWidget::DockLeft,getMainDockWidget(),25);
+			break;
+			
+		case DOCK_RIGHT:
+		default:
+			tw->place(KDockWidget::DockRight,getMainDockWidget(),25);
+			break;
+	}
+}
+
+void KTorrent::removeToolWidget(QWidget* w)
+{
+	if (!w) return;
+	
+	w->reparent(0,QPoint());
+	deleteToolWindow(w);
 }
 
 const TorrentInterface* KTorrent::getCurrentTorrent() const
@@ -806,42 +844,14 @@ const TorrentInterface* KTorrent::getCurrentTorrent() const
 	return m_view->getCurrentTC();
 }
 
-KTorrentView* KTorrent::getCurrentView()
-{
-	return m_tabs->currentPageIndex() == 1 ? m_seedView : m_view;
-}
-
-PanelView KTorrent::getCurrentPanel()
-{
-	if(!isVisible())
-		return NO_VIEW;
-	
-	return (PanelView) m_tabs->currentPageIndex();
-}
-
-void KTorrent::currentTabChanged(QWidget* )
-{
-	if(m_tabs->currentPageIndex() == 1)
-	{
-		m_seedView->onSelectionChanged(); // trigger an updateActions signal
-		currentSeedChanged(m_seedView->getCurrentTC());
-	}
-	
-	if(m_tabs->currentPageIndex() == 0)
-	{
-		m_view->onSelectionChanged(); // trigger an updateActions signal
-		currentDownloadChanged(m_view->getCurrentTC());
-	}
-}
-
 void KTorrent::queueAction()
 {
-	getCurrentView()->queueSlot();
+	m_view->queueSlot();
 }
 
 void KTorrent::checkDataIntegrity()
 {
-	getCurrentView()->checkDataIntegrity();
+	m_view->checkDataIntegrity();
 }
 
 QString KTorrent::getStatusInfo() 
@@ -866,14 +876,51 @@ QString KTorrent::getStatusDHT()
 
 QCStringList KTorrent::getTorrentInfo(kt::TorrentInterface* tc)
 {
-	QCStringList torrentinfo;
-	if(!tc)
-		return torrentinfo;
-	if(tc->getStats().completed)
-		torrentinfo = m_seedView->getTorrentInfo(tc);
-	else
-		torrentinfo = m_view->getTorrentInfo(tc);
-	return torrentinfo;
+	return m_view->getTorrentInfo(tc);
+}
+
+void KTorrent::showIPFilter()
+{
+	IPFilterWidget ipf(this);
+	ipf.exec();
+}
+
+void KTorrent::tabClosePressed()
+{
+	KMdiChildView* child = activeWindow();
+	if (!child)
+		return;
+	
+	QMap<QWidget*,Tab>::iterator i = m_tab_map.begin();
+	while (i != m_tab_map.end())
+	{
+		if (i.data().child == child)
+		{
+			kt::CloseTabListener* ctl = i.data().ctl;
+			ctl->tabCloseRequest(i.key());
+			return; 
+		}
+		i++;
+	}
+}
+
+void KTorrent::currentTabChanged(QWidget* )
+{
+	KMdiChildView* child = activeWindow();
+	if (!child)
+		return;
+	
+	QMap<QWidget*,Tab>::iterator i = m_tab_map.begin();
+	while (i != m_tab_map.end())
+	{
+		if (i.data().child == child)
+		{
+			m_close_cur_tab->setEnabled(i.data().ctl != 0);
+			return;
+		}
+		i++;
+	}
+	m_close_cur_tab->setEnabled(false);
 }
 
 #include "ktorrent.moc"
