@@ -27,6 +27,7 @@
 #include <kmessagebox.h>
 #include <kstandarddirs.h>
 #include <kapplication.h>
+#include <kio/job.h>
 #include <kio/netaccess.h>
 #include <qregexp.h>
 
@@ -35,8 +36,10 @@
 #include <torrent/globals.h>
 #include <util/error.h>
 #include <util/fileops.h>
+#include <util/waitjob.h>
 #include <torrent/torrentcreator.h>
 #include <torrent/server.h>
+#include <torrent/authenticationmonitor.h>
 #include <util/functions.h>
 #include <torrent/ipblocklist.h>
 #include <kademlia/dhtbase.h>
@@ -51,13 +54,20 @@
 #include "fileselectdlg.h"
 #include "scandialog.h"
 
+
+#include <util/profiler.h>
+
+
 using namespace bt;
 using namespace kt;
 
+const Uint32 CORE_UPDATE_INTERVAL = 250;
+
 
 
 KTorrentCore::KTorrentCore(kt::GUIInterface* gui) : max_downloads(0),keep_seeding(true),pman(0)
 {
+	UpdateCurrentTime();
 	qman = new QueueManager();
 	data_dir = Settings::tempDir();
 	bool dd_not_exist = !bt::Exists(data_dir);
@@ -78,9 +88,14 @@ KTorrentCore::KTorrentCore(kt::GUIInterface* gui) : max_downloads(0),keep_seedin
 	// 	downloads.setAutoDelete(true);
 
 	connect(&update_timer,SIGNAL(timeout()),this,SLOT(update()));
-	update_timer.start(250);
-
+	update_timer.start(CORE_UPDATE_INTERVAL);
+	
 	Uint16 port = Settings::port();
+	if (port == 0)
+	{
+		port = 6881;
+		Settings::setPort(6881);
+	}
 	Uint16 i = 0;
 	do
 	{
@@ -94,7 +109,7 @@ KTorrentCore::KTorrentCore(kt::GUIInterface* gui) : max_downloads(0),keep_seedin
 		if (port != port + i - 1)
 			KMessageBox::information(0,
 			                         i18n("Specified port (%1) is unavailable or in"
-			                              " use by another application. KTorrent is bound to port %2.")
+			                              " use by another application. KTorrent is now using port %2.")
 			                         .arg(port).arg(port + i - 1));
 
 		Out(SYS_GEN|LOG_NOTICE) << "Bound to port " << (port + i - 1) << endl;
@@ -102,7 +117,8 @@ KTorrentCore::KTorrentCore(kt::GUIInterface* gui) : max_downloads(0),keep_seedin
 	else
 	{
 		KMessageBox::error(0,
-		                   i18n("Cannot bind to port %1 or the 10 following ports.").arg(port));
+			i18n("KTorrent is unable to accept connections because the ports %1 to %2 are "
+			   "already in use by another program.").arg(port).arg(port + i - 1));
 		Out(SYS_GEN|LOG_IMPORTANT) << "Cannot find free port" << endl;
 	}
 
@@ -123,45 +139,62 @@ void KTorrentCore::loadPlugins()
 	pman->loadPluginList();
 }
 
-void KTorrentCore::load(const QString & target,const QString & dir,bool silently)
+bool KTorrentCore::init(TorrentControl* tc,bool silently)
+{
+	connectSignals(tc);
+	qman->append(tc);
+	if (tc->getStats().multi_file_torrent && !silently)
+	{
+		FileSelectDlg dlg;
+
+		if (dlg.execute(tc) != QDialog::Accepted)
+		{
+			if (!tc->hasExistingFiles())
+				remove(tc,true);
+			else
+				remove(tc,false);
+			return false;
+		}
+	}
+	
+	tc->createFiles();
+		
+	if (tc->hasExistingFiles())
+	{
+		ScanDialog* scan_dlg = new ScanDialog(this,true);
+		scan_dlg->show();
+		scan_dlg->execute(tc,true);
+	//	scan_dlg->deleteLater();
+	}
+		
+	tc->setPreallocateDiskSpace(true);
+
+	if(Settings::maxRatio()>0)
+        	tc->setMaxShareRatio(Settings::maxRatio()); 
+	
+	torrentAdded(tc);
+	qman->torrentAdded(tc);
+		
+	return true;
+}
+
+bool KTorrentCore::load(const QByteArray & data,const QString & dir,bool silently, const KURL& url)
 {
-	TorrentControl* tc = 0;
 	QString tdir = findNewTorrentDir();
+	TorrentControl* tc = 0;
 	try
 	{
-		Out(SYS_GEN|LOG_NOTICE) << "Loading file " << target << endl;
+		Out(SYS_GEN|LOG_NOTICE) << "Loading torrent from data " << endl;
 		tc = new TorrentControl();
-		tc->init(qman, target, tdir, dir, 
-		         Settings::useSaveDir() ? Settings::saveDir() : QString());
-
-		connectSignals(tc);
-		qman->append(tc);
-		if (tc->getStats().multi_file_torrent && !silently)
-		{
-			FileSelectDlg dlg;
-
-			if (dlg.execute(tc) != QDialog::Accepted)
-			{
-				if (!tc->hasExistingFiles())
-					remove(tc,true);
-				else
-					remove(tc,false);
-				return;
-			}
-		}
+		tc->init(qman, data, tdir, dir, 
+				 Settings::useSaveDir() ? Settings::saveDir() : QString());
 		
-		if (tc->hasExistingFiles())
-		{
-			ScanDialog* scan_dlg = new ScanDialog(true);
-			scan_dlg->show();
-			scan_dlg->execute(tc,true);
-			scan_dlg->deleteLater();
-		}
+		if(!init(tc,silently))
+			loadingFinished(url, false, true);
+		else
+			loadingFinished(url, true, false);
 		
-		tc->setPreallocateDiskSpace(true);
-	
-		torrentAdded(tc);
-		qman->torrentAdded(tc);
+		return true;
 	}
 	catch (bt::Error & err)
 	{
@@ -171,60 +204,146 @@ void KTorrentCore::load(const QString & target,const QString & dir,bool silently
 		// delete tdir if necesarry
 		if (bt::Exists(tdir))
 			bt::Delete(tdir,true);
+		
+		loadingFinished(url, false, false);
+		
+		return false;
 	}
 }
 
-void KTorrentCore::load(const KURL& url)
+bool KTorrentCore::load(const QString & target,const QString & dir,bool silently)
 {
-	QString target;
-	// download the contents
-	if (KIO::NetAccess::download(url,target,0))
+	QString tdir = findNewTorrentDir();
+	TorrentControl* tc = 0;
+	try
+	{
+		Out(SYS_GEN|LOG_NOTICE) << "Loading file " << target << endl;
+		tc = new TorrentControl();
+		tc->init(qman, target, tdir, dir, 
+			 Settings::useSaveDir() ? Settings::saveDir() : QString());
+		
+		init(tc,silently);
+		return true;
+	}
+	catch (bt::Error & err)
+	{
+		KMessageBox::error(0,err.toString());
+		delete tc;
+		tc = 0;
+		// delete tdir if necesarry
+		if (bt::Exists(tdir))
+			bt::Delete(tdir,true);
+		return false;
+	}
+}
+
+void KTorrentCore::downloadFinished(KIO::Job *job)
+{
+	KIO::StoredTransferJob* j = (KIO::StoredTransferJob*)job;
+	int err = j->error();
+	if (err == KIO::ERR_USER_CANCELED)
+	{
+		loadingFinished(j->url(),false,true);
+		return;
+	}
+	
+	if (err)
+	{
+		loadingFinished(j->url(),false,false);
+		j->showErrorDialog(0);
+	}
+	else
 	{
 		// load in the file (target is always local)
 		QString dir = Settings::saveDir();
 		if (!Settings::useSaveDir())
 			dir = KFileDialog::getExistingDirectory(QString::null, 0,
-			                                        i18n("Select Folder to Save To"));
-
-
-		if (dir != QString::null)
-		{
-			load(target,dir,false);
-		}
-		// and remove the temp file
-		KIO::NetAccess::removeTempFile(target);
-	}
-	else
-	{
-		KMessageBox::error(0,KIO::NetAccess::lastErrorString(),i18n("Error"));
+				i18n("Select Folder to Save To"));
+	
+		if (dir != QString::null && load(j->data(),dir,false, j->url()))
+			loadingFinished(j->url(),true,false);
+		else
+			loadingFinished(j->url(),false,true);
 	}
 }
 
-void KTorrentCore::loadSilently(const KURL& url)
+void KTorrentCore::load(const KURL& url)
 {
-	QString target;
-	// download the contents
-	if (KIO::NetAccess::download(url,target,0))
+	if (url.isLocalFile())
+	{
+		QString path = url.path(); 
+		QString dir = Settings::saveDir();
+		if (!Settings::useSaveDir())
+			dir = KFileDialog::getExistingDirectory(QString::null, 0,
+				i18n("Select folder for data of %1").arg(url.prettyURL()));
+	
+		if (dir != QString::null && load(path,dir,false))
+			loadingFinished(url,true,false);
+		else
+			loadingFinished(url,false,true);
+	}
+	else
+	{
+		KIO::Job* j = KIO::storedGet(url,false,true);
+		connect(j,SIGNAL(result(KIO::Job*)),this,SLOT(downloadFinished( KIO::Job* )));
+	}
+}
+
+void KTorrentCore::downloadFinishedSilently(KIO::Job *job)
+{
+	KIO::StoredTransferJob* j = (KIO::StoredTransferJob*)job;
+	int err = j->error();
+	if (err == KIO::ERR_USER_CANCELED)
+	{
+		loadingFinished(j->url(),false,true);
+		return;
+	}
+	
+	if (err)
+	{
+		loadingFinished(j->url(),false,false);
+	}
+	else
 	{
 		// load in the file (target is always local)
 		QString dir = Settings::saveDir();
 		if (!Settings::useSaveDir())
 		{
-			KMessageBox::error(0,i18n("You need to have default save directory selected to load torrents silently."),i18n("Error"));
+			loadingFinished(j->url(),false,false);
 		}
 		else
 		{
-			if (dir != QString::null)
-			{
-				load(target,dir,true);
-			}
+			if (dir != QString::null && load(j->data(),dir,true,j->url()))
+				loadingFinished(j->url(),true,false);
+			else
+				loadingFinished(j->url(),false,false);
 		}
-		// and remove the temp file
-		KIO::NetAccess::removeTempFile(target);
+	}
+}
+
+void KTorrentCore::loadSilently(const KURL& url)
+{
+	if (url.isLocalFile())
+	{
+		QString path = url.path(); 
+		QString dir = Settings::saveDir();
+		if (!Settings::useSaveDir())
+		{
+			Out(SYS_GEN|LOG_NOTICE) << "Cannot load " << path << " silently, default save location not set !" << endl;
+			Out(SYS_GEN|LOG_NOTICE) << "Using home directory instead !" << endl;
+			dir = QDir::homeDirPath();
+		}
+	
+		if (dir != QString::null && load(path,dir,true))
+			loadingFinished(url,true,false);
+		else
+			loadingFinished(url,false,true);
 	}
 	else
 	{
-		KMessageBox::error(0,KIO::NetAccess::lastErrorString(),i18n("Error"));
+		// download to a random file in tmp
+		KIO::Job* j = KIO::storedGet(url,false,true);
+		connect(j,SIGNAL(result(KIO::Job*)),this,SLOT(downloadFinishedSilently( KIO::Job* )));
 	}
 }
 
@@ -267,6 +386,9 @@ void KTorrentCore::loadExistingTorrent(const QString & tor_dir)
 	if (!idir.endsWith(bt::DirSeparator()))
 		idir += bt::DirSeparator();
 	
+	if (!bt::Exists(idir + "torrent"))
+		return;
+	
 	try
 	{
 		tc = new TorrentControl();
@@ -356,6 +478,7 @@ void KTorrentCore::torrentFinished(kt::TorrentInterface* tc)
 
 void KTorrentCore::setKeepSeeding(bool ks)
 {
+	keep_seeding = ks;
 	qman->setKeepSeeding(ks);
 }
 
@@ -363,8 +486,21 @@ void KTorrentCore::onExit()
 {
 	// stop timer to prevent updates during wait
 	update_timer.stop();
-	//pman->saveConfigFile(KGlobal::dirs()->saveLocation("data","ktorrent") + "plugins");
-	// 	downloads.clear();
+	// stop all authentications going on
+	AuthenticationMonitor::instance().clear();
+	// shutdown the server
+	Globals::instance().shutdownServer();
+	
+	WaitJob* job = new WaitJob(5000);
+	qman->onExit(job);
+	// wait for completion of stopped events
+	if (job->needToWait())
+	{
+		WaitJob::execute(job);
+	}
+	else
+		delete job;
+	
 	qman->clear();
 	pman->unloadAll(false);
 }
@@ -377,7 +513,7 @@ bool KTorrentCore::changeDataDir(const QString & new_dir)
 		// do nothing if new and old dir are the same
 		if (KURL(data_dir) == KURL(new_dir) || data_dir == (new_dir + bt::DirSeparator()))
 		{
-			update_timer.start(100);
+			update_timer.start(CORE_UPDATE_INTERVAL);
 			return true;
 		}
 
@@ -392,8 +528,9 @@ bool KTorrentCore::changeDataDir(const QString & new_dir)
 			nd += DirSeparator();
 
 		Out() << "Switching to datadir " << nd << endl;
-		// keep track of all TorrentInterface's which have succesfully
-		// moved to the new data dir
+		
+		qman->setPausedState(true);
+		
 		QPtrList<kt::TorrentInterface> succes;
 
 		QPtrList<kt::TorrentInterface>::iterator i = qman->begin();
@@ -407,7 +544,8 @@ bool KTorrentCore::changeDataDir(const QString & new_dir)
 				// set back the old data_dir in Settings
 				Settings::setTempDir(data_dir);
 				Settings::self()->writeConfig();
-				update_timer.start(100);
+				qman->setPausedState(false);
+				update_timer.start(CORE_UPDATE_INTERVAL);
 				return false;
 			}
 			else
@@ -417,13 +555,14 @@ bool KTorrentCore::changeDataDir(const QString & new_dir)
 			i++;
 		}
 		data_dir = nd;
-		update_timer.start(100);
+		qman->setPausedState(false);
+		update_timer.start(CORE_UPDATE_INTERVAL);
 		return true;
 	}
 	catch (bt::Error & e)
 	{
 		Out(SYS_GEN|LOG_IMPORTANT) << "Error : " << e.toString() << endl;
-		update_timer.start(100);
+		update_timer.start(CORE_UPDATE_INTERVAL);
 		return false;
 	}
 }
@@ -438,7 +577,7 @@ void KTorrentCore::rollback(const QPtrList<kt::TorrentInterface> & succes)
 		(*i)->rollback();
 		i++;
 	}
-	update_timer.start(100);
+	update_timer.start(CORE_UPDATE_INTERVAL);
 }
 
 void KTorrentCore::startAll(int type)
@@ -453,27 +592,47 @@ void KTorrentCore::stopAll(int type)
 
 void KTorrentCore::update()
 {
-	Globals::instance().getServer().update();
-
+	bt::UpdateCurrentTime();
+	KT_PROF_START("core");
+	AuthenticationMonitor::instance().update();
+	
 	QPtrList<kt::TorrentInterface>::iterator i = qman->begin();
 	//Uint32 down_speed = 0;
 	while (i != qman->end())
 	{
+		bool finished = false;
 		kt::TorrentInterface* tc = *i;
-		if (tc->getStats().running)
+		if (tc->isCheckingData(finished))
 		{
-			tc->update();
+			if (finished)
+				tc->afterDataCheck();
+		}
+		else if (tc->getStats().running)
+		{
+			// see if we need to do a auto data check
+			if (Settings::autoRecheck() && tc->getStats().num_corrupted_chunks >= Settings::maxCorruptedBeforeRecheck())
+			{
+				Out(SYS_GEN|LOG_IMPORTANT) << "Doing an automatic data check on " 
+						<< tc->getStats().torrent_name << endl;
+		
+				ScanDialog* scan_dlg = new ScanDialog(this,false);
+				scan_dlg->show();
+				scan_dlg->execute(tc,true);
+			}
+			else
+			{
+				tc->update();
+			}
 		}
 		i++;
 	}
-	
-	
+	KT_PROF_END();
 }
 
 void KTorrentCore::makeTorrent(const QString & file,const QStringList & trackers,
                                int chunk_size,const QString & name,
                                const QString & comments,bool seed,
-							   const QString & output_file,bool priv_tor,KProgress* prog)
+							   const QString & output_file,bool priv_tor,KProgress* prog, bool decentralized)
 {
 	QString tdir;
 	try
@@ -481,7 +640,7 @@ void KTorrentCore::makeTorrent(const QString & file,const QStringList & trackers
 		if (chunk_size < 0)
 			chunk_size = 256;
 
-		bt::TorrentCreator mktor(file,trackers,chunk_size,name,comments,priv_tor);
+		bt::TorrentCreator mktor(file,trackers,chunk_size,name,comments,priv_tor, decentralized);
 		prog->setTotalSteps(mktor.getNumChunks());
 		Uint32 ns = 0;
 		while (!mktor.calculateHash())
@@ -599,6 +758,7 @@ bt::QueueManager* KTorrentCore::getQueueManager()
 void KTorrentCore::torrentSeedAutoStopped( kt::TorrentInterface * tc )
 {
 	qman->startNext();
+	maxShareRatioReached(tc);
 }
 
 int KTorrentCore::getMaxUploadSpeed()
@@ -769,10 +929,10 @@ void KTorrentCore::aboutToBeStarted(kt::TorrentInterface* tc,bool & ret)
 	
 	if (tc->getStats().multi_file_torrent)
 	{
-		QString msg = i18n("Several data files of the torrent \"%1\" are missing, do you want to recreate them, or do you want to not download them ?").arg(tc->getStats().torrent_name);
+		QString msg = i18n("Several data files of the torrent \"%1\" are missing, do you want to recreate them, or do you want to not download them?").arg(tc->getStats().torrent_name);
 					
 		int ret = KMessageBox::warningYesNoCancelList(0,msg,missing,QString::null,
-				KGuiItem(i18n("Recreate")),KGuiItem(i18n("Do not download")));
+				KGuiItem(i18n("Recreate")),KGuiItem(i18n("Do Not Download")));
 		if (ret == KMessageBox::Yes)
 		{
 			try
@@ -782,7 +942,7 @@ void KTorrentCore::aboutToBeStarted(kt::TorrentInterface* tc,bool & ret)
 			}
 			catch (bt::Error & e)
 			{
-				KMessageBox::error(0,i18n("Cannot recreate missing files : %1").arg(e.toString()));
+				KMessageBox::error(0,i18n("Cannot recreate missing files: %1").arg(e.toString()));
 				tc->handleError(i18n("Data files are missing"));
 				ret = false;
 			}
@@ -796,7 +956,7 @@ void KTorrentCore::aboutToBeStarted(kt::TorrentInterface* tc,bool & ret)
 			}
 			catch (bt::Error & e)
 			{
-				KMessageBox::error(0,i18n("Cannot deselect missing files : %1").arg(e.toString()));
+				KMessageBox::error(0,i18n("Cannot deselect missing files: %1").arg(e.toString()));
 				tc->handleError(i18n("Data files are missing"));
 				ret = false;
 			}
@@ -809,8 +969,8 @@ void KTorrentCore::aboutToBeStarted(kt::TorrentInterface* tc,bool & ret)
 	}
 	else
 	{
-		QString msg = i18n("The file where the data is saved of the torrent \"%1\" is missing, do you want to recreate it ?").arg(tc->getStats().torrent_name);
-		int ret = KMessageBox::warningYesNo(0,msg);
+		QString msg = i18n("The file where the data is saved of the torrent \"%1\" is missing, do you want to recreate it?").arg(tc->getStats().torrent_name);
+		int ret = KMessageBox::warningYesNo(0,msg, i18n("Recreate"), KStdGuiItem::cancel());
 		if (ret == KMessageBox::Yes)
 		{
 			try
@@ -819,7 +979,7 @@ void KTorrentCore::aboutToBeStarted(kt::TorrentInterface* tc,bool & ret)
 			}
 			catch (bt::Error & e)
 			{
-				KMessageBox::error(0,i18n("Cannot recreate data file : %1").arg(e.toString()));
+				KMessageBox::error(0,i18n("Cannot recreate data file: %1").arg(e.toString()));
 				tc->handleError(i18n("Data file is missing"));
 				ret = false;
 			}
@@ -833,6 +993,12 @@ void KTorrentCore::aboutToBeStarted(kt::TorrentInterface* tc,bool & ret)
 	
 }
 
+void KTorrentCore::emitCorruptedData(kt::TorrentInterface* tc)
+{
+	corruptedData(tc);
+	
+}
+
 void KTorrentCore::connectSignals(kt::TorrentInterface* tc)
 {
 	connect(tc,SIGNAL(finished(kt::TorrentInterface*)),
@@ -843,6 +1009,33 @@ void KTorrentCore::connectSignals(kt::TorrentInterface* tc)
 			this, SLOT(torrentSeedAutoStopped( kt::TorrentInterface* )));
 	connect(tc,SIGNAL(aboutToBeStarted( kt::TorrentInterface*,bool & )),
 			this, SLOT(aboutToBeStarted( kt::TorrentInterface*,bool & )));
+	connect(tc,SIGNAL(corruptedDataFound( kt::TorrentInterface* )),
+			this, SLOT(emitCorruptedData( kt::TorrentInterface* )));
+	connect(qman, SIGNAL(queuingNotPossible( kt::TorrentInterface* )),
+			this, SLOT(enqueueTorrentOverMaxRatio( kt::TorrentInterface* )));
+}
+
+float KTorrentCore::getGlobalMaxShareRatio() const
+{
+	return Settings::maxRatio();
+}
+
+void KTorrentCore::enqueueTorrentOverMaxRatio(kt::TorrentInterface* tc)
+{
+	emit queuingNotPossible(tc);
+}
+
+
+void KTorrentCore::doDataCheck(kt::TorrentInterface* tc)
+{
+	bool dummy = false;
+	if (tc->isCheckingData(dummy))
+		return;
+	
+	ScanDialog* scan_dlg = new ScanDialog(this,false);
+	scan_dlg->setCaption(i18n("Checking Data Integrity"));
+	scan_dlg->show();
+	scan_dlg->execute(tc,false);
 }
 
 #include "ktorrentcore.moc"
