@@ -29,17 +29,20 @@
 #include <util/bitset.h>
 #include <util/functions.h>
 #include <util/fileops.h>
+#include <util/waitjob.h>
 #include <interfaces/functions.h>
 #include <interfaces/trackerslist.h>
 #include <datachecker/singledatachecker.h>
 #include <datachecker/multidatachecker.h>
 #include <datachecker/datacheckerlistener.h>
+#include <datachecker/datacheckerthread.h>
 #include <migrate/ccmigrate.h>
 #include <migrate/cachemigrate.h>
 #include <kademlia/dhtbase.h>
+
 #include "downloader.h"
 #include "uploader.h"
-#include "tracker.h"
+#include "peersourcemanager.h"
 #include "chunkmanager.h"
 #include "torrent.h"
 #include "peermanager.h"
@@ -61,6 +64,10 @@
 #include "statsfile.h"
 #include "announcelist.h"
 #include "preallocationthread.h"
+#include "timeestimator.h"
+
+#include <util/profiler.h>
+
 
 using namespace kt;
 
@@ -70,8 +77,9 @@ namespace bt
 
 
 	TorrentControl::TorrentControl()
-	: tor(0),tracker(0),cman(0),pman(0),down(0),up(0),choke(0),tmon(0),prealloc(false), last_announce(0)
+	: tor(0),psman(0),cman(0),pman(0),down(0),up(0),choke(0),tmon(0),prealloc(false)
 	{
+		istats.last_announce = 0;
 		stats.imported_bytes = 0;
 		stats.trk_bytes_downloaded = 0;
 		stats.trk_bytes_uploaded = 0;
@@ -80,22 +88,29 @@ namespace bt
 		stats.stopped_by_error = false;
 		stats.session_bytes_downloaded = 0;
 		stats.session_bytes_uploaded = 0;
-		session_bytes_uploaded = 0;
+		istats.session_bytes_uploaded = 0;
 		old_datadir = QString::null;
 		stats.status = NOT_STARTED;
 		stats.autostart = true;
 		stats.user_controlled = false;
 		stats.priv_torrent = false;
-		running_time_dl = running_time_ul = 0;
-		prev_bytes_dl = 0;
-		prev_bytes_ul = 0;
-		trk_prev_bytes_dl = trk_prev_bytes_ul = 0;
-		io_error = false;
-		priority = 0;
-		maxShareRatio = 0.00f;
-		custom_output_name = false;
+		stats.seeders_connected_to = stats.seeders_total = 0;
+		stats.leechers_connected_to = stats.leechers_total = 0;
+		istats.running_time_dl = istats.running_time_ul = 0;
+		istats.prev_bytes_dl = 0;
+		istats.prev_bytes_ul = 0;
+		istats.trk_prev_bytes_dl = istats.trk_prev_bytes_ul = 0;
+		istats.io_error = false;
+		istats.priority = 0;
+		istats.maxShareRatio = 0.00f;
+		istats.custom_output_name = false;
 		updateStats();
 		prealoc_thread = 0;
+		dcheck_thread = 0;
+		istats.dht_on = false;
+		stats.num_corrupted_chunks = 0;
+		
+		m_eta = new TimeEstimator(this);
 	}
 
 
@@ -113,17 +128,18 @@ namespace bt
 		delete up;
 		delete cman;
 		delete pman;
-		delete tracker;
+		delete psman;
 		delete tor;
+		delete m_eta;
 	}
 
 	void TorrentControl::update()
 	{
-		// do not update during critical operation mode or when we are data checking
-		if (Globals::instance().inCriticalOperationMode() || stats.status == kt::CHECKING_DATA)
+		UpdateCurrentTime();
+		if (stats.status == kt::CHECKING_DATA)
 			return;
 		
-		if (io_error)
+		if (istats.io_error)
 		{
 			stop(false);
 			emit stoppedByError(this, error_msg);
@@ -159,6 +175,7 @@ namespace bt
 				return; // preallocation still going on, so just return
 		}
 		
+
 		try
 		{
 			// first update peermanager
@@ -166,20 +183,24 @@ namespace bt
 			bool comp = stats.completed;
 
 			// then the downloader and uploader
-			up->update(choke->getOptimisticlyUnchokedPeerID());
-			//if (!completed)
+			up->update(choke->getOptimisticlyUnchokedPeerID());			
 			down->update();
 
-			stats.completed = cman->chunksLeft() == 0;
+			stats.completed = cman->completed();
 			if (stats.completed && !comp)
 			{
-				// download has just been completed
-				tracker->completed();
 				pman->killSeeders();
+				pman->killUninterested();
 				QDateTime now = QDateTime::currentDateTime();
-				running_time_dl += time_started_dl.secsTo(now);
+				istats.running_time_dl += istats.time_started_dl.secsTo(now);
 				updateStatusMsg();
 				updateStats();
+				
+				// download has just been completed
+				// only sent completed to tracker when we have all chunks (so no excluded chunks)
+				if (cman->haveAllChunks())
+					psman->completed();
+				
 				finished(this);
 			}
 			else if (!stats.completed && comp)
@@ -187,22 +208,29 @@ namespace bt
 				// restart download if necesarry
 				// when user selects that files which were previously excluded,
 				// should now be downloaded
-				tracker->start();
-				time_started_dl = QDateTime::currentDateTime();
+				if (!psman->isStarted())
+					psman->start();
+				else
+					psman->manualUpdate();
+				istats.last_announce = bt::GetCurrentTime();
+				istats.time_started_dl = QDateTime::currentDateTime();
 			}
 			updateStatusMsg();
-
+			
 			// get rid of dead Peers
 			Uint32 num_cleared = pman->clearDeadPeers();
 			
 			// we may need to update the choker
 			if (choker_update_timer.getElapsedSinceUpdate() >= 10000 || num_cleared > 0)
 			{
-				// also get rid of seeders when download is finished
+				// also get rid of seeders & uninterested when download is finished
 				// no need to keep them around, but also no need to do this
 				// every update, so once every 10 seconds is fine
 				if (stats.completed)
+				{
 					pman->killSeeders();
+					pman->killUninterested();
+				}
 				
 				doChoking();
 				choker_update_timer.update();
@@ -219,25 +247,32 @@ namespace bt
 
 			// Update DownloadCap
 			updateStats();
+
 			if (stats.download_rate > 0)
 				stalled_timer.update();
 			
 			// do a manual update if we are stalled for more then 2 minutes
 			// we do not do this for private torrents
-			if (stalled_timer.getElapsedSinceUpdate() > 120000 && stats.bytes_left > 0 &&
+			if (stalled_timer.getElapsedSinceUpdate() > 120000 && !stats.completed &&
 				!stats.priv_torrent)
 			{
-				Out() << "Stalled for to long, time to get some fresh blood" << endl;
-				tracker->manualUpdate();
+				Out(SYS_TRK|LOG_NOTICE) << "Stalled for too long, time to get some fresh blood" << endl;
+				psman->manualUpdate();
 				stalled_timer.update();
 			}
 			
-			if(overMaxRatio())
-			{
-				stop(true);
+			if(overMaxRatio()) 
+			{ 
+				if(istats.priority!=0) //if it's queued make sure to dequeue it 
+				{
+					setPriority(0);
+					stats.user_controlled = true;
+				}
+                 
+				stop(true); 
 				emit seedingAutoStopped(this);
-			}
-			
+            } 
+
 		}
 		catch (Error & e)
 		{
@@ -247,12 +282,11 @@ namespace bt
 
 	void TorrentControl::onIOError(const QString & msg)
 	{
-		Out() << "Error : " << msg << endl;
+		Out(SYS_DIO|LOG_IMPORTANT) << "Error : " << msg << endl;
 		stats.stopped_by_error = true;
 		stats.status = ERROR;
 		error_msg = msg;
-		short_error_msg = msg;
-		io_error = true;
+		istats.io_error = true;
 	}
 
 	void TorrentControl::start()
@@ -262,9 +296,9 @@ namespace bt
 			return;
 
 		stats.stopped_by_error = false;
-		io_error = false;
+		istats.io_error = false;
 		try
-		{
+		{	
 			bool ret = true;
 			aboutToBeStarted(this,ret);
 			if (!ret)
@@ -287,11 +321,8 @@ namespace bt
 			throw;
 		}
 		
-		time_started_ul = time_started_dl = QDateTime::currentDateTime();
-		trk_prev_bytes_dl = stats.bytes_downloaded,
-		trk_prev_bytes_ul = stats.bytes_uploaded,
-		stats.trk_bytes_downloaded = 0;
-		stats.trk_bytes_uploaded = 0;
+		istats.time_started_ul = istats.time_started_dl = QDateTime::currentDateTime();
+		resetTrackerStats();
 		
 		if (prealloc)
 		{
@@ -308,8 +339,9 @@ namespace bt
 	
 	void TorrentControl::continueStart()
 	{
-		// continues start after the prealoc_thread has finished preallocation
+		// continues start after the prealoc_thread has finished preallocation	
 		pman->start();
+		pman->loadPeerList(datadir + "peer_list");
 		try
 		{
 			down->loadDownloads(datadir + "current_chunks");
@@ -330,18 +362,19 @@ namespace bt
 		
 		
 		stalled_timer.update();
-		tracker->start();
+		psman->start();
+		istats.last_announce = bt::GetCurrentTime();
 		stalled_timer.update();
 	}
 		
 
-	void TorrentControl::stop(bool user)
+	void TorrentControl::stop(bool user,WaitJob* wjob)
 	{
 		QDateTime now = QDateTime::currentDateTime();
 		if(!stats.completed)
-			running_time_dl += time_started_dl.secsTo(now);
-		running_time_ul += time_started_ul.secsTo(now);
-		time_started_ul = time_started_dl = now;
+			istats.running_time_dl += istats.time_started_dl.secsTo(now);
+		istats.running_time_ul += istats.time_started_ul.secsTo(now);
+		istats.time_started_ul = istats.time_started_dl = now;
 		
 		// stop preallocation thread if necesarry
 		if (prealoc_thread)
@@ -366,7 +399,7 @@ namespace bt
 	
 		if (stats.running)
 		{
-			tracker->stop();
+			psman->stop(wjob);
 
 			if (tmon)
 				tmon->stopped();
@@ -379,7 +412,7 @@ namespace bt
 			{
 				// print out warning in case of failure
 				// it doesn't corrupt the data, so just a couple of lost chunks
-				Out() << "Warning : " << e.toString() << endl;
+				Out(SYS_GEN|LOG_NOTICE) << "Warning : " << e.toString() << endl;
 			}
 			
 			down->clearDownloads();
@@ -390,6 +423,7 @@ namespace bt
 				stats.autostart = false;
 			}
 		}
+		pman->savePeerList(datadir + "peer_list");
 		pman->stop();
 		pman->closeAllConnections();
 		pman->clearDeadPeers();
@@ -413,23 +447,14 @@ namespace bt
 				tmon->peerAdded(pman->getPeer(i));
 		}
 	}
-
+	
+	
 	void TorrentControl::init(QueueManager* qman,
 							  const QString & torrent,
 							  const QString & tmpdir,
 							  const QString & ddir,
 							  const QString & default_save_dir)
 	{
-		datadir = tmpdir;
-		stats.completed = false;
-		stats.running = false;
-		if (!datadir.endsWith(DirSeparator()))
-			datadir += DirSeparator();
-
-		outputdir = ddir.stripWhiteSpace();
-		if (outputdir.length() > 0 && !outputdir.endsWith(DirSeparator()))
-			outputdir += DirSeparator();
-
 		// first load the torrent file
 		tor = new Torrent();
 		try
@@ -444,26 +469,87 @@ namespace bt
 					" The torrent is probably corrupt or is not a torrent file."));
 		}
 		
+		initInternal(qman,tmpdir,ddir,default_save_dir,torrent.startsWith(tmpdir));
+		
+		// copy torrent in tor dir
+		QString tor_copy = datadir + "torrent";
+		if (tor_copy != torrent)
+		{
+			bt::CopyFile(torrent,tor_copy);
+		}
+		
+	}
+	
+	
+	void TorrentControl::init(QueueManager* qman, const QByteArray & data,const QString & tmpdir,
+							  const QString & ddir,const QString & default_save_dir)
+	{
+		// first load the torrent file
+		tor = new Torrent();
+		try
+		{
+			tor->load(data,false);
+		}
+		catch (...)
+		{
+			delete tor;
+			tor = 0;
+			throw Error(i18n("An error occurred while loading the torrent."
+					" The torrent is probably corrupt or is not a torrent file."));
+		}
+		
+		initInternal(qman,tmpdir,ddir,default_save_dir,true);
+		// copy data into torrent file
+		QString tor_copy = datadir + "torrent";
+		QFile fptr(tor_copy);
+		if (!fptr.open(IO_WriteOnly))
+			throw Error(i18n("Unable to create %1 : %2")
+					.arg(tor_copy).arg(fptr.errorString()));
+	
+		fptr.writeBlock(data.data(),data.size());
+	}
+	
+	void TorrentControl::checkExisting(QueueManager* qman)
+	{
 		// check if we haven't already loaded the torrent
 		// only do this when qman isn't 0
 		if (qman && qman->allreadyLoaded(tor->getInfoHash()))
 		{
-			if (tor->getAnnounceList())
+			if (!stats.priv_torrent)
 			{
-				qman->mergeAnnounceList(tor->getInfoHash(),tor->getAnnounceList());
+				qman->mergeAnnounceList(tor->getInfoHash(),tor->getTrackerList());
+
+				throw Error(i18n("You are already downloading this torrent %1, the list of trackers of both torrents has been merged.").arg(tor->getNameSuggestion()));
 			}
-			else 
+			else
 			{
-				qman->addTrackerURL(tor->getInfoHash(),tor->getTrackerURL(true));
+				throw Error(i18n("You are already downloading the torrent %1")
+						.arg(tor->getNameSuggestion()));
 			}
-			throw Error(i18n("You are already downloading this torrent, the list of trackers of both torrents has been merged."));
 		}
+	}
+	
+	void TorrentControl::setupDirs(const QString & tmpdir,const QString & ddir)
+	{
+		datadir = tmpdir;
+		
+		if (!datadir.endsWith(DirSeparator()))
+			datadir += DirSeparator();
+
+		outputdir = ddir.stripWhiteSpace();
+		if (outputdir.length() > 0 && !outputdir.endsWith(DirSeparator()))
+			outputdir += DirSeparator();
 		
 		if (!bt::Exists(datadir))
 		{
 			bt::MakeDir(datadir);
 		}
-
+	}
+	
+	void TorrentControl::setupStats()
+	{
+		stats.completed = false;
+		stats.running = false;
 		stats.torrent_name = tor->getNameSuggestion();
 		stats.multi_file_torrent = tor->isMultiFile();
 		stats.total_bytes = tor->getFileLength();
@@ -473,21 +559,65 @@ namespace bt
 		StatsFile st(datadir + "stats");
 		if (st.hasKey("CUSTOM_OUTPUT_NAME") && st.readULong("CUSTOM_OUTPUT_NAME") == 1)
 		{
-			custom_output_name = true;
+			istats.custom_output_name = true;
 		}
 		
 		// load outputdir if outputdir is null
 		if (outputdir.isNull() || outputdir.length() == 0)
 			loadOutputDir();
-					
-		// copy torrent in temp dir
-		QString tor_copy = datadir + "torrent";
+	}
+	
+	void TorrentControl::setupData(const QString & ddir)
+	{
+		// create PeerManager and Tracker
+		pman = new PeerManager(*tor);
+		//Out() << "Tracker url " << url << " " << url.protocol() << " " << url.prettyURL() << endl;
+		psman = new PeerSourceManager(this,pman);
+		connect(psman,SIGNAL(statusChanged( const QString& )),
+				this,SLOT(trackerStatusChanged( const QString& )));
 
-		if (tor_copy != torrent)
-		{
-			bt::CopyFile(torrent,tor_copy);
-		}
-		else
+
+		// Create chunkmanager, load the index file if it exists
+		// else create all the necesarry files
+		cman = new ChunkManager(*tor,datadir,outputdir,istats.custom_output_name);
+		// outputdir is null, see if the cache has figured out what it is
+		if (outputdir.length() == 0)
+			outputdir = cman->getDataDir();
+		
+		// store the outputdir into the output_path variable, so others can access it	
+		
+		connect(cman,SIGNAL(updateStats()),this,SLOT(updateStats()));
+		if (bt::Exists(datadir + "index"))
+			cman->loadIndexFile();
+
+		stats.completed = cman->completed();
+
+		// create downloader,uploader and choker
+		down = new Downloader(*tor,*pman,*cman);
+		connect(down,SIGNAL(ioError(const QString& )),
+				this,SLOT(onIOError(const QString& )));
+		up = new Uploader(*cman,*pman);
+		choke = new Choker(*pman,*cman);
+
+
+		connect(pman,SIGNAL(newPeer(Peer* )),this,SLOT(onNewPeer(Peer* )));
+		connect(pman,SIGNAL(peerKilled(Peer* )),this,SLOT(onPeerRemoved(Peer* )));
+		connect(cman,SIGNAL(excluded(Uint32, Uint32 )),down,SLOT(onExcluded(Uint32, Uint32 )));
+		connect(cman,SIGNAL(included( Uint32, Uint32 )),down,SLOT(onIncluded( Uint32, Uint32 )));
+		connect(cman,SIGNAL(corrupted( Uint32 )),this,SLOT(corrupted( Uint32 )));
+	}
+	
+	void TorrentControl::initInternal(QueueManager* qman,
+									  const QString & tmpdir,
+									  const QString & ddir,
+									  const QString & default_save_dir,
+									  bool first_time)
+	{
+		checkExisting(qman);
+		setupDirs(tmpdir,ddir);
+		setupStats();
+
+		if (!first_time)
 		{
 			// if we do not need to copy the torrent, it is an existing download and we need to see
 			// if it is not an old download
@@ -503,49 +633,7 @@ namespace bt
 						.arg(tor->getNameSuggestion()).arg(err.toString()));
 			}
 		}
-
-
-		// create PeerManager and Tracker
-		pman = new PeerManager(*tor);
-		KURL url = tor->getTrackerURL(true);
-		//Out() << "Tracker url " << url << " " << url.protocol() << " " << url.prettyURL() << endl;
-		tracker = new Tracker(this,tor->getInfoHash(),tor->getPeerID());
-		connect(tracker,SIGNAL(error()),this,SLOT(trackerResponseError()));
-		connect(tracker,SIGNAL(dataReady()),this,SLOT(trackerResponse()));
-
-
-		// Create chunkmanager, load the index file if it exists
-		// else create all the necesarry files
-		cman = new ChunkManager(*tor,datadir,outputdir,custom_output_name);
-		// outputdir is null, see if the cache has figured out what it is
-		if (outputdir.length() == 0)
-			outputdir = cman->getDataDir();
-		
-		// store the outputdir into the output_path variable, so others can access it	
-		
-		connect(cman,SIGNAL(updateStats()),this,SLOT(updateStats()));
-		if (bt::Exists(datadir + "index"))
-			cman->loadIndexFile();
-
-	
-		// as a sanity check make sure all files are created properly the first time we load this torrent
-		if (!stats.completed && !ddir.isNull())
-			cman->createFiles();
-
-		stats.completed = cman->chunksLeft() == 0;
-
-		// create downloader,uploader and choker
-		down = new Downloader(*tor,*pman,*cman);
-		connect(down,SIGNAL(ioError(const QString& )),
-				this,SLOT(onIOError(const QString& )));
-		up = new Uploader(*cman,*pman);
-		choke = new Choker(*pman,*cman);
-
-
-		connect(pman,SIGNAL(newPeer(Peer* )),this,SLOT(onNewPeer(Peer* )));
-		connect(pman,SIGNAL(peerKilled(Peer* )),this,SLOT(onPeerRemoved(Peer* )));
-		connect(cman,SIGNAL(excluded(Uint32, Uint32 )),down,SLOT(onExcluded(Uint32, Uint32 )));
-		connect(cman,SIGNAL(included( Uint32, Uint32 )),down,SLOT(onIncluded( Uint32, Uint32 )));
+		setupData(ddir);
 
 		updateStatusMsg();
 
@@ -555,7 +643,7 @@ namespace bt
 		{
 			Uint64 db = down->bytesDownloaded();
 			Uint64 cb = down->getDownloadedBytesOfCurrentChunksFile(datadir + "current_chunks");
-			prev_bytes_dl = db + cb;
+			istats.prev_bytes_dl = db + cb;
 				
 		//	Out() << "Downloaded : " << kt::BytesToString(db) << endl;
 		//	Out() << "current_chunks : " << kt::BytesToString(cb) << endl;
@@ -564,70 +652,43 @@ namespace bt
 		{
 			// print out warning in case of failure
 			Out() << "Warning : " << e.toString() << endl;
-			prev_bytes_dl = down->bytesDownloaded();
+			istats.prev_bytes_dl = down->bytesDownloaded();
 		}
 		
 		loadStats();
 		updateStats();
 		saveStats();
 		stats.output_path = cman->getOutputPath();
+		if (stats.output_path.isNull())
+		{
+			cman->createFiles();
+			stats.output_path = cman->getOutputPath();
+		}
 		Out() << "OutputPath = " << stats.output_path << endl;
-		
-		AnnounceList* list = tor->getAnnounceList();
-		if(!list)
-			list = tor->createAnnounceList();
-		
-		list->setDatadir(datadir);
 	}
+	
 
-	void TorrentControl::trackerResponse()
-	{
-		try
-		{
-			tracker->updateData(pman);
-			updateStatusMsg();
-			stats.trackerstatus = i18n("OK");
-		}
-		catch (Error & e)
-		{
-			Out(SYS_TRK|LOG_IMPORTANT) << "Error : " << e.toString() << endl;
-			stats.trackerstatus = i18n("Invalid response");
-			tracker->handleError();
-		}
-	}
-
-	void TorrentControl::trackerResponseError()
-	{
-		Out(SYS_TRK|LOG_IMPORTANT) << "Tracker Response Error" << endl;
-		stats.trackerstatus = i18n("Unreachable");
-		tracker->handleError();
-	}
 
 	bool TorrentControl::announceAllowed()
 	{
-		if(last_announce == 0)
+		if(istats.last_announce == 0)
 			return true;
 		
-		return bt::GetCurrentTime() - last_announce >= 60 * 1000;
+		if (psman && psman->getNumFailures() == 0)
+			return bt::GetCurrentTime() - istats.last_announce >= 60 * 1000;
+		else
+			return true;
 	}
 	
 	void TorrentControl::updateTracker()
 	{
 		if (stats.running && announceAllowed())
 		{
-			tracker->manualUpdate();
-			last_announce = bt::GetCurrentTime();
+			psman->manualUpdate();
+			istats.last_announce = bt::GetCurrentTime();
 		}
 	}
 
-	KURL TorrentControl::getTrackerURL(bool prev_success) const
-	{
-		if (tor)
-			return tor->getTrackerURL(prev_success);
-		else
-			return KURL();
-	}
-
 	void TorrentControl::onNewPeer(Peer* p)
 	{
 		connect(p,SIGNAL(gotPortPacket( const QString&, Uint16 )),
@@ -651,8 +712,16 @@ namespace bt
 		if (!stats.completed)
 			p->getPacketWriter().sendInterested();
 		
-		if (p->isDHTSupported() && !stats.priv_torrent)
-			p->getPacketWriter().sendPort(Globals::instance().getDHT().getPort());
+		if (!stats.priv_torrent)
+		{
+			if (p->isDHTSupported())
+				p->getPacketWriter().sendPort(Globals::instance().getDHT().getPort());
+			else
+				// WORKAROUND so we can contact µTorrent's DHT
+				// They do not properly support the standard and do not turn on
+				// the DHT bit in the handshake, so we just ping each peer by default.
+				p->emitPortPacket();
+		}
 		
 		if (tmon)
 			tmon->peerAdded(p);
@@ -668,85 +737,50 @@ namespace bt
 
 	void TorrentControl::doChoking()
 	{
-		choke->update(cman->bytesLeft() == 0,stats);
+		choke->update(stats.completed,stats);
 	}
 
 	bool TorrentControl::changeDataDir(const QString & new_dir)
 	{
-		// new_dir doesn't contain the torX/ part
-		// so first get that and append it to new_dir
-		int dd = datadir.findRev(DirSeparator(),datadir.length() - 2,false);
-		QString tor = datadir.mid(dd + 1,datadir.length() - 2 - dd);
-
-
-		// make sure nd ends with a /
-		QString nd = new_dir + tor;
-		if (!nd.endsWith(DirSeparator()))
-			nd += DirSeparator();
-
-		Out() << datadir << " -> " << nd << endl;
-
-		int ok_calls = 0;
-		try
+		int pos = datadir.findRev(bt::DirSeparator(),-2);
+		if (pos == -1)
 		{
-			if (!bt::Exists(nd))
-				bt::MakeDir(nd);
-
-			// now try to move all the files :
-			// first the torrent
-			bt::Move(datadir + "torrent",nd);
-			ok_calls++;
-			// then the index
-			bt::Move(datadir + "index",nd);
-			ok_calls++;
-			// then the cache
-			bt::Move(datadir + "cache",nd);
-			ok_calls++;
-
-			// tell the chunkmanager that the datadir has changed
-			cman->changeDataDir(nd);
-		}
-		catch (...)
-		{
-			// move the torrent back
-			if (ok_calls >= 1)
-				bt::Move(nd + "torrent",datadir,true);
-			if (ok_calls >= 2)
-				bt::Move(nd + "index",datadir,true);
+			Out(SYS_GEN|LOG_DEBUG) << "Could not find torX part in " << datadir << endl;
 			return false;
 		}
-
-		// we don't move the current_chunks file
-		// it will be recreated anyway
-		// now delete the old directory
-		bt::Delete(datadir,true);
-
-		old_datadir = datadir;
-		datadir = nd;
+		
+		QString ndatadir = new_dir + datadir.mid(pos + 1);
+		
+		Out(SYS_GEN|LOG_DEBUG) << datadir << " -> " << ndatadir << endl;
+		try
+		{
+			bt::Move(datadir,ndatadir);
+			old_datadir = datadir;
+			datadir = ndatadir;
+		}
+		catch (Error & err)
+		{
+			Out(SYS_GEN|LOG_IMPORTANT) << "Could not move " << datadir << " to " << ndatadir << endl;
+			return false;
+		}
+		
+		cman->changeDataDir(datadir);
 		return true;
 	}
 
 
 	void TorrentControl::rollback()
 	{
-		if (old_datadir.isNull())
-			return;
-
-		// recreate it
-		if (!bt::Exists(old_datadir))
-			bt::MakeDir(old_datadir,true);
-
-		// move back files
-		bt::Move(datadir + "torrent",old_datadir,true);
-		bt::Move(datadir + "cache",old_datadir,true);
-		bt::Move(datadir + "index",old_datadir,true);
-		cman->changeDataDir(old_datadir);
-
-		// delete new
-		bt::Delete(datadir,true);
-
-		datadir = old_datadir;
-		old_datadir = QString::null;
+		try
+		{
+			bt::Move(datadir,old_datadir);
+			datadir = old_datadir;
+			cman->changeDataDir(datadir);
+		}
+		catch (Error & err)
+		{
+			Out(SYS_GEN|LOG_IMPORTANT) << "Could not move " << datadir << " to " << old_datadir << endl;
+		}
 	}
 
 	void TorrentControl::updateStatusMsg()
@@ -757,14 +791,17 @@ namespace bt
 			stats.status = kt::NOT_STARTED;
 		else if(!stats.running && !stats.user_controlled)
 			stats.status = kt::QUEUED;
+		else if (!stats.running && stats.completed && overMaxRatio())
+			stats.status = kt::SEEDING_COMPLETE;
 		else if (!stats.running && stats.completed)
-			stats.status = kt::COMPLETE;
+			stats.status = kt::DOWNLOAD_COMPLETE;
 		else if (!stats.running)
 			stats.status = kt::STOPPED;
 		else if (stats.running && stats.completed)
 			stats.status = kt::SEEDING;
-		else if (stats.running)
-			stats.status = down->downloadRate() > 0 ?
+		else if (stats.running) 
+			// protocol messages are also included in speed calculation, so lets not compare with 0
+			stats.status = down->downloadRate() > 100 ?
 					kt::DOWNLOADING : kt::STALLED;
 	}
 
@@ -791,6 +828,14 @@ namespace bt
 		else
 			return cman->getExcludedBitSet();
 	}
+	
+	const BitSet & TorrentControl::onlySeedChunksBitSet() const
+	{
+		if (!cman)
+			return BitSet::null;
+		else
+			return cman->getOnlySeedBitSet();
+	}
 
 	void TorrentControl::saveStats()
 	{
@@ -806,22 +851,29 @@ namespace bt
 		if (stats.running)
 		{
 			QDateTime now = QDateTime::currentDateTime();
-			st.write("RUNNING_TIME_DL",QString("%1").arg(running_time_dl + time_started_dl.secsTo(now)));
-			st.write("RUNNING_TIME_UL",QString("%1").arg(running_time_ul + time_started_ul.secsTo(now)));
+			st.write("RUNNING_TIME_DL",QString("%1").arg(istats.running_time_dl + istats.time_started_dl.secsTo(now)));
+			st.write("RUNNING_TIME_UL",QString("%1").arg(istats.running_time_ul + istats.time_started_ul.secsTo(now)));
 		}
 		else
 		{
-			st.write("RUNNING_TIME_DL", QString("%1").arg(running_time_dl));
-			st.write("RUNNING_TIME_UL", QString("%1").arg(running_time_ul));
+			st.write("RUNNING_TIME_DL", QString("%1").arg(istats.running_time_dl));
+			st.write("RUNNING_TIME_UL", QString("%1").arg(istats.running_time_ul));
 		}
 		
-		st.write("PRIORITY", QString("%1").arg(priority));
+		st.write("PRIORITY", QString("%1").arg(istats.priority));
 		st.write("AUTOSTART", QString("%1").arg(stats.autostart));
 		st.write("IMPORTED", QString("%1").arg(stats.imported_bytes));
-		st.write("CUSTOM_OUTPUT_NAME",custom_output_name ? "1" : "0");
-		st.write("MAX_RATIO", QString("%1").arg(maxShareRatio,0,'f',2));
+		st.write("CUSTOM_OUTPUT_NAME",istats.custom_output_name ? "1" : "0");
+		st.write("MAX_RATIO", QString("%1").arg(istats.maxShareRatio,0,'f',2));
 		st.write("RESTART_DISK_PREALLOCATION",prealloc ? "1" : "0");
 		
+		if(!stats.priv_torrent)
+		{
+			//save dht and pex 
+			st.write("DHT", isFeatureEnabled(kt::DHT_FEATURE) ? "1" : "0");
+			st.write("UT_PEX", isFeatureEnabled(kt::UT_PEX_FEATURE) ? "1" : "0");
+		}
+		
 		st.writeSync();
 	}
 
@@ -832,29 +884,39 @@ namespace bt
 		Uint64 val = st.readUint64("UPLOADED");
 		// stats.session_bytes_uploaded will be calculated based upon prev_bytes_ul
 		// seeing that this will change here, we need to save it 
-		session_bytes_uploaded = stats.session_bytes_uploaded; 
-		prev_bytes_ul = val;
+		istats.session_bytes_uploaded = stats.session_bytes_uploaded; 
+		istats.prev_bytes_ul = val;
 		up->setBytesUploaded(val);
 		
-		this->running_time_dl = st.readULong("RUNNING_TIME_DL");
-		this->running_time_ul = st.readULong("RUNNING_TIME_UL");
+		this->istats.running_time_dl = st.readULong("RUNNING_TIME_DL");
+		this->istats.running_time_ul = st.readULong("RUNNING_TIME_UL");
 		outputdir = st.readString("OUTPUTDIR").stripWhiteSpace();
 		if (st.hasKey("CUSTOM_OUTPUT_NAME") && st.readULong("CUSTOM_OUTPUT_NAME") == 1)
 		{
-			custom_output_name = true;
+			istats.custom_output_name = true;
 		}
 		
 		setPriority(st.readInt("PRIORITY"));
-		stats.user_controlled = priority == 0 ? true : false;
+		stats.user_controlled = istats.priority == 0 ? true : false;
 		stats.autostart = st.readBoolean("AUTOSTART");
 		
 		stats.imported_bytes = st.readUint64("IMPORTED");
 		float rat = st.readFloat("MAX_RATIO");
-		maxShareRatio = rat;
+		istats.maxShareRatio = rat;
 		if (st.hasKey("RESTART_DISK_PREALLOCATION"))
 			prealloc = st.readString("RESTART_DISK_PREALLOCATION") == "1";
 		
-		return;
+		if (!stats.priv_torrent)
+		{
+			if(st.hasKey("DHT"))
+				istats.dht_on = st.readBoolean("DHT");
+			else
+				istats.dht_on = true;
+			
+			setFeatureEnabled(kt::DHT_FEATURE,istats.dht_on);
+			if (st.hasKey("UT_PEX"))
+				setFeatureEnabled(kt::UT_PEX_FEATURE,st.readBoolean("UT_PEX"));
+		}
 	}
 
 	void TorrentControl::loadOutputDir()
@@ -866,7 +928,7 @@ namespace bt
 		outputdir = st.readString("OUTPUTDIR").stripWhiteSpace();
 		if (st.hasKey("CUSTOM_OUTPUT_NAME") && st.readULong("CUSTOM_OUTPUT_NAME") == 1)
 		{
-			custom_output_name = true;
+			istats.custom_output_name = true;
 		}
 	}
 
@@ -884,8 +946,8 @@ namespace bt
 
 	Uint32 TorrentControl::getTimeToNextTrackerUpdate() const
 	{
-		if (tracker)
-			return tracker->getTimeToNextUpdate();
+		if (psman)
+			return psman->getTimeToNextUpdate();
 		else
 			return 0;
 	}
@@ -897,35 +959,39 @@ namespace bt
 		stats.upload_rate = up && stats.running ? up->uploadRate() : 0;
 		stats.download_rate = down && stats.running ? down->downloadRate() : 0;
 		stats.bytes_left = cman ? cman->bytesLeft() : 0;
+		stats.bytes_left_to_download = cman ? cman->bytesLeftToDownload() : 0;
 		stats.bytes_uploaded = up ? up->bytesUploaded() : 0;
 		stats.bytes_downloaded = down ? down->bytesDownloaded() : 0;
-		stats.total_chunks = cman ? cman->getNumChunks() : 0;
-		stats.num_chunks_downloaded = cman ? cman->getNumChunks() - cman->chunksExcluded() - cman->chunksLeft() : 0;
+		stats.total_chunks = tor ? tor->getNumChunks() : 0;
+		stats.num_chunks_downloaded = cman ? cman->chunksDownloaded() : 0;
 		stats.num_chunks_excluded = cman ? cman->chunksExcluded() : 0;
 		stats.chunk_size = tor ? tor->getChunkSize() : 0;
 		stats.total_bytes_to_download = (tor && cman) ?	tor->getFileLength() - cman->bytesExcluded() : 0;
+		stats.max_share_ratio = istats.maxShareRatio;
 		
-		if (stats.bytes_downloaded >= prev_bytes_dl)
-			stats.session_bytes_downloaded = stats.bytes_downloaded - prev_bytes_dl;
+		
+		
+		if (stats.bytes_downloaded >= istats.prev_bytes_dl)
+			stats.session_bytes_downloaded = stats.bytes_downloaded - istats.prev_bytes_dl;
 		else
 			stats.session_bytes_downloaded = 0;
 					
-		if (stats.bytes_uploaded >= prev_bytes_ul)
-			stats.session_bytes_uploaded = (stats.bytes_uploaded - prev_bytes_ul) + session_bytes_uploaded;
+		if (stats.bytes_uploaded >= istats.prev_bytes_ul)
+			stats.session_bytes_uploaded = (stats.bytes_uploaded - istats.prev_bytes_ul) + istats.session_bytes_uploaded;
 		else
-			stats.session_bytes_uploaded = session_bytes_uploaded;
+			stats.session_bytes_uploaded = istats.session_bytes_uploaded;
 		/*
 			Safety check, it is possible that stats.bytes_downloaded gets subtracted in Downloader.
-			Which can cause stats.bytes_downloaded to be smaller the trk_prev_bytes_dl.
+			Which can cause stats.bytes_downloaded to be smaller the istats.trk_prev_bytes_dl.
 			This can screw up your download ratio.
 		*/
-		if (stats.bytes_downloaded >= trk_prev_bytes_dl)
-			stats.trk_bytes_downloaded = stats.bytes_downloaded - trk_prev_bytes_dl;
+		if (stats.bytes_downloaded >= istats.trk_prev_bytes_dl)
+			stats.trk_bytes_downloaded = stats.bytes_downloaded - istats.trk_prev_bytes_dl;
 		else
 			stats.trk_bytes_downloaded = 0;
 		
-		if (stats.bytes_uploaded >= trk_prev_bytes_ul)
-			stats.trk_bytes_uploaded = stats.bytes_uploaded - trk_prev_bytes_ul;
+		if (stats.bytes_uploaded >= istats.trk_prev_bytes_ul)
+			stats.trk_bytes_uploaded = stats.bytes_uploaded - istats.trk_prev_bytes_ul;
 		else
 			stats.trk_bytes_uploaded = 0;
 		
@@ -937,7 +1003,7 @@ namespace bt
 	{
 		total = 0;
 		connected_to = 0;
-		if (!pman || !tracker)
+		if (!pman || !psman)
 			return;
 
 		for (Uint32 i = 0;i < pman->getNumConnectedPeers();i++)
@@ -945,7 +1011,7 @@ namespace bt
 			if (pman->getPeer(i)->isSeeder())
 				connected_to++;
 		}
-		total = tracker->getNumSeeders();
+		total = psman->getNumSeeders();
 		if (total == 0)
 			total = connected_to;
 	}
@@ -954,7 +1020,7 @@ namespace bt
 	{
 		total = 0;
 		connected_to = 0;
-		if (!pman || !tracker)
+		if (!pman || !psman)
 			return;
 
 		for (Uint32 i = 0;i < pman->getNumConnectedPeers();i++)
@@ -962,7 +1028,7 @@ namespace bt
 			if (!pman->getPeer(i)->isSeeder())
 				connected_to++;
 		}
-		total = tracker->getNumLeechers();
+		total = psman->getNumLeechers();
 		if (total == 0)
 			total = connected_to;
 	}
@@ -970,17 +1036,17 @@ namespace bt
 	Uint32 TorrentControl::getRunningTimeDL() const
 	{
 		if (!stats.running || stats.completed)
-			return running_time_dl;
+			return istats.running_time_dl;
 		else
-			return running_time_dl + time_started_dl.secsTo(QDateTime::currentDateTime());
+			return istats.running_time_dl + istats.time_started_dl.secsTo(QDateTime::currentDateTime());
 	}
 
 	Uint32 TorrentControl::getRunningTimeUL() const
 	{
 		if (!stats.running)
-			return running_time_ul;
+			return istats.running_time_ul;
 		else
-			return running_time_ul + time_started_ul.secsTo(QDateTime::currentDateTime());
+			return istats.running_time_ul + istats.time_started_ul.secsTo(QDateTime::currentDateTime());
 	}
 
 	Uint32 TorrentControl::getNumFiles() const
@@ -1048,7 +1114,7 @@ namespace bt
 	
 	void TorrentControl::setPriority(int p)
 	{
-		priority = p;
+		istats.priority = p;
 		stats.user_controlled = p == 0 ? true : false;
 		if(p)
 			stats.status = kt::QUEUED;
@@ -1062,11 +1128,14 @@ namespace bt
 	{
 		if(ratio == 1.00f)
 		{
-			if(maxShareRatio != ratio)
-				maxShareRatio = ratio;
+			if(istats.maxShareRatio != ratio)
+				istats.maxShareRatio = ratio;
 		}
 		else
-			maxShareRatio = ratio;
+			istats.maxShareRatio = ratio;
+		
+		if(stats.completed && !stats.running && !stats.user_controlled && (kt::ShareRatio(stats) >= istats.maxShareRatio))
+			setPriority(0); //dequeue it
 		
 		saveStats();
 		emit maxRatioChanged(this);
@@ -1074,9 +1143,9 @@ namespace bt
 	
 	bool TorrentControl::overMaxRatio()
 	{
-		if(stats.completed && stats.bytes_uploaded != 0 && stats.bytes_downloaded != 0 && maxShareRatio > 0)
+		if(stats.completed && stats.bytes_uploaded != 0 && stats.bytes_downloaded != 0 && istats.maxShareRatio > 0)
 		{
-			if(kt::ShareRatio(stats) >= maxShareRatio)
+			if(kt::ShareRatio(stats) >= istats.maxShareRatio)
 				return true;
 		}
 		
@@ -1090,8 +1159,10 @@ namespace bt
 		{
 			case kt::NOT_STARTED :
 				return i18n("Not started");
-			case kt::COMPLETE :
-				return i18n("Completed");
+			case kt::DOWNLOAD_COMPLETE :
+				return i18n("Download completed");
+			case kt::SEEDING_COMPLETE :
+				return i18n("Seeding completed");
 			case kt::SEEDING :
 				return i18n("Seeding");
 			case kt::DOWNLOADING:
@@ -1114,16 +1185,12 @@ namespace bt
 
 	TrackersList* TorrentControl::getTrackersList()
 	{
-		AnnounceList* list = tor->getAnnounceList();
-		if(!list)
-			list = tor->createAnnounceList();
-		
-		return list;
+		return psman;
 	}
 	
-	TrackersList* TorrentControl::createTrackersList()
+	const TrackersList* TorrentControl::getTrackersList() const 
 	{
-		return tor->createAnnounceList();
+		return psman;
 	}
 
 	void TorrentControl::onPortPacket(const QString & ip,Uint16 port)
@@ -1132,36 +1199,84 @@ namespace bt
 			Globals::instance().getDHT().portRecieved(ip,port);
 	}
 	
-	void TorrentControl::doDataCheck(bt::DataCheckerListener* lst,bool auto_import)
+	void TorrentControl::startDataCheck(bt::DataCheckerListener* lst,bool auto_import)
 	{
 		if (stats.status == kt::ALLOCATING_DISKSPACE)
 			return;
 		
+		
 		DataChecker* dc = 0;
 		stats.status = kt::CHECKING_DATA;
+		stats.num_corrupted_chunks = 0; // reset the number of corrupted chunks found
 		if (stats.multi_file_torrent)
 			dc = new MultiDataChecker();
 		else
 			dc = new SingleDataChecker();
 	
 		dc->setListener(lst);
-		dc->check(stats.output_path,*tor,datadir + "dnd" + bt::DirSeparator());
+		
+		dcheck_thread = new DataCheckerThread(dc,stats.output_path,*tor,datadir + "dnd" + bt::DirSeparator());
+		
+		// dc->check(stats.output_path,*tor,datadir + "dnd" + bt::DirSeparator());
+		dcheck_thread->start();
+	}
+	
+	void TorrentControl::afterDataCheck()
+	{
+		DataChecker* dc = dcheck_thread->getDataChecker();
+		DataCheckerListener* lst = dc->getListener();
+		
+		bool err = !dcheck_thread->getError().isNull();
+		if (err)
+		{
+			// show a queued error message when an error has occurred
+			KMessageBox::queuedMessageBox(0,KMessageBox::Error,dcheck_thread->getError());
+			lst->stop();
+		}
 		
 		if (lst && !lst->isStopped())
 		{
 			down->dataChecked(dc->getDownloaded());
-			// update chunk manager
+				// update chunk manager
 			cman->dataChecked(dc->getDownloaded());
-			if (auto_import)
+			if (lst->isAutoImport())
 			{
 				down->recalcDownloaded();
 				stats.imported_bytes = down->bytesDownloaded();
+				if (cman->haveAllChunks())
+					stats.completed = true;
+			}
+			else
+			{
+				Uint64 downloaded = stats.bytes_downloaded;
+				down->recalcDownloaded();
+				updateStats();
+				if (stats.bytes_downloaded > downloaded)
+					stats.imported_bytes = stats.bytes_downloaded - downloaded;
+				 
+				if (cman->haveAllChunks())
+					stats.completed = true;
 			}
 		}
-		delete dc;
+			
+		stats.status = kt::NOT_STARTED;
 		// update the status
 		updateStatusMsg();
 		updateStats();
+		if (lst)
+			lst->finished();
+		delete dcheck_thread;
+		dcheck_thread = 0;
+	}
+	
+	bool TorrentControl::isCheckingData(bool & finished) const
+	{
+		if (dcheck_thread)
+		{
+			finished = !dcheck_thread->isRunning();
+			return true;
+		}
+		return false;
 	}
 	
 	bool TorrentControl::hasExistingFiles() const
@@ -1224,6 +1339,117 @@ namespace bt
 	{
 		cman->deleteDataFiles();
 	}
+	
+	const bt::SHA1Hash & TorrentControl::getInfoHash() const
+	{
+		return tor->getInfoHash();
+	}
+	
+	void TorrentControl::resetTrackerStats()
+	{
+		istats.trk_prev_bytes_dl = stats.bytes_downloaded,
+		istats.trk_prev_bytes_ul = stats.bytes_uploaded,
+		stats.trk_bytes_downloaded = 0;
+		stats.trk_bytes_uploaded = 0;
+	}
+	
+	void TorrentControl::trackerStatusChanged(const QString & ns)
+	{
+		stats.trackerstatus = ns;
+	}
+	
+	void TorrentControl::addPeerSource(kt::PeerSource* ps)
+	{
+		if (psman)
+			psman->addPeerSource(ps);
+	}
+	
+	void TorrentControl::removePeerSource(kt::PeerSource* ps)
+	{
+		if (psman)
+			psman->removePeerSource(ps);
+	}
+	
+	void TorrentControl::corrupted(Uint32 chunk)
+	{
+		// make sure we will redownload the chunk
+		down->corrupted(chunk);
+		if (stats.completed)
+			stats.completed = false;
+		
+		// emit signal to show a systray message
+		stats.num_corrupted_chunks++;
+		corruptedDataFound(this);
+	}
+	
+	Uint32 TorrentControl::getETA()
+	{
+		return m_eta->estimate();
+	}
+	
+	
+	
+	const bt::PeerID & TorrentControl::getOwnPeerID() const
+	{
+		return tor->getPeerID();
+	}
+	
+	
+	bool TorrentControl::isFeatureEnabled(TorrentFeature tf)
+	{
+		switch (tf)
+		{
+		case kt::DHT_FEATURE:
+			return psman->dhtStarted();
+		case kt::UT_PEX_FEATURE:
+			return pman->isPexEnabled();
+		default:
+			return false;
+		}
+	}
+		
+	void TorrentControl::setFeatureEnabled(TorrentFeature tf,bool on)
+	{
+		switch (tf)
+		{
+		case kt::DHT_FEATURE:
+			if (on)
+			{
+				if(!stats.priv_torrent)
+				{
+					psman->addDHT();
+					istats.dht_on = psman->dhtStarted();
+					saveStats();
+				}
+			}
+			else
+			{
+				psman->removeDHT();
+				istats.dht_on = false;
+				saveStats();
+			}
+			break;
+		case kt::UT_PEX_FEATURE:
+			if (on)
+			{
+				if (!stats.priv_torrent && !pman->isPexEnabled())
+				{
+					pman->setPexEnabled(true);
+				}
+			}
+			else
+			{
+				pman->setPexEnabled(false);
+			}
+			break;
+		}	
+	}
+	
+	void TorrentControl::createFiles()
+	{
+		cman->createFiles();
+		stats.output_path = cman->getOutputPath();
+	}
 }
 
 #include "torrentcontrol.moc"
